@@ -16,10 +16,14 @@ test_that("Predict", {
   mod <- fnb.gaussian(x, y)
   df_mod <- fnb.gaussian(df, y)
 
+  expect_error(fnb.gaussian(x[1:3,], y[1:3]))
+
   predictions <- predict(mod, x, type = "raw")
+  risky_predictions <- predict(mod, x, type = "raw", check=FALSE)
   df_predictions <- predict(df_mod, df, type = "raw")
 
   expect_equal(sum(round(abs(predictions-df_predictions), digits = 12)), 0)
+  expect_equal(sum(round(abs(predictions-risky_predictions), digits = 12)), 0)
 
   classification <- predict(mod, x, type = "class")
   expect_equal(as.factor(mod$levels[max.col(predictions)]), classification)
@@ -30,7 +34,7 @@ test_that("Predict", {
 
   dropped_x <- x[,1:3]
   mod <- fnb.gaussian(dropped_x, y)
-  alt_predictions <- predict(mod, x, type = "raw")
+  alt_predictions <- predict(mod, x, type = "raw", silent=TRUE)
 
   expect_equal(sum(round(abs(dropped_predictions-alt_predictions), digits = 12)), 0)
 
@@ -41,7 +45,7 @@ test_that("Predict", {
   x <- cbind(x, x[,1, drop=FALSE])
   colnames(x)[5] <- "womo"
 
-  new_predictions <- predict(mod, x, type = "raw")
+  new_predictions <- predict(mod, x, type = "raw", silent=TRUE)
   expect_equal(sum(round(abs(predictions-new_predictions), digits = 12)), 0)
 
   # All new columns is same as all 0
